#include "pico/stdlib.h"
#include <stdio.h>
#include <math.h>

#include "pico/time.h"
#include "hardware/irq.h"


#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_gpio\include\hardware\gpio.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_sync\include\hardware\sync.h>
#include "hardware/resets.h"
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_pwm\include\hardware\pwm.h>


#define GPIO4 4     // pin 6    
#define PWMpin1 2    // GPIO2 pin 4
#define PWMpin2 3    // GPIO3 pin 5

#define PWMclkDiv 1000

#define ms 1e-3
#define us 1e-6
#define ns 1e-9
#define NN 1000

//-----------------------------------------------------------------------------------

static void approxSleepUs(int n){
  for(int k=0 ; k<n ; k++){
    for( int kk=0 ; kk<24 ; kk++){
      asm volatile("nop");
      }
    }
  }
  
static void approxSleepMs(int n){
  for(int k=0 ; k<n ; k++){
    for( int kk=0 ; kk<10 ; kk++){
      approxSleepUs(100) ;
      }
    }
  }

static volatile void approxSleepUsTest(){
  printf("atleastSleepUsTest()\n") ;
  save_and_disable_interrupts() ;
  while(1){
    sio_hw->gpio_set =(1ul << GPIO4) ;  
	  approxSleepUs(1000) ;
    //sleep_ms(1) ;
    sio_hw->gpio_clr =(1ul << GPIO4) ;  
    approxSleepUs(1000) ;
  //  printf(".") ;
  }
} 

//---------------------------------------------------------------
volatile uint32_t DDS1phase=0 ; // DDS phase value
volatile uint32_t DDS1diff=0 ; // DDS phase delta
volatile uint32_t DDS2phase=0 ; // DDS phase value
volatile uint32_t DDS2diff=0 ; // DDS phase delta

volatile int DDSsinTab[256] ;
volatile int DDScosTab[256] ;
volatile int DDStriTab[256] ;

float triangle(float x){
  if(x<=M_PI/2){ return x/(M_PI/2.0) ; }
  if(x<=M_PI){ return triangle(M_PI-x) ; }
  return -triangle(2*M_PI-x) ;
  }

//------------------------------------------------------------------------------------------------------
void PWMsetup(){    
  gpio_set_function(PWMpin1, GPIO_FUNC_PWM);
  gpio_set_function(PWMpin2, GPIO_FUNC_PWM);
  uint slice_num1 = pwm_gpio_to_slice_num(PWMpin1);
  uint slice_num2 = pwm_gpio_to_slice_num(PWMpin2);
  pwm_config config1 = pwm_get_default_config();
  pwm_config_set_clkdiv(&config1, 1.f);
  pwm_init(slice_num1, &config1, true);
  pwm_set_wrap(slice_num1,(PWMclkDiv-1) );
  pwm_set_wrap(slice_num2,(PWMclkDiv-1) );
  float fPWM=125e6/PWMclkDiv ;
  printf("fPWM=%10.1f\n",fPWM) ;
  pwm_set_gpio_level(PWMpin1, 200 );
  pwm_set_gpio_level(PWMpin2, 500 );
  }


double fSample ;

bool TimerCallback(struct repeating_timer *t) {
  gpio_put(GPIO4, 1);
  pwm_set_gpio_level(PWMpin1, DDSsinTab[DDS1phase>>24] );
  pwm_set_gpio_level(PWMpin2, DDStriTab[DDS2phase>>24] );
  DDS1phase += DDS1diff ; 
  DDS2phase += DDS2diff ; 
  gpio_put(GPIO4, 0);
  }

void setupDDS(){
  fSample=10000 ;
  double DDS1frq=20.0 ;
  double DDS2frq=10.0 ;
  DDS1diff=(int)( 4294967296.0*DDS1frq/fSample) ;
  DDS2diff=(int)( 4294967296.0*DDS2frq/fSample) ;

  for(int k=0 ; k<256 ; k++){
    double phi=2*M_PI*k/256.0 ;
    DDScosTab[k]=500+450*cos(phi) ;
    DDSsinTab[k]=500+450*sin(phi) ;
    DDStriTab[k]=500+450*triangle(phi) ;
    }

  
  }
  
//----------------------------------------------------------------------------------------  


void abort(){
  printf("abort..\n") ;
  printf("\n") ;
  while(1){} ;
  }


int main() {
  stdio_init_all();
  gpio_init(GPIO4);
  gpio_set_dir(GPIO4, GPIO_OUT);
   
  busy_wait_us_32(200000) ;
  printf("//\n") ;
  printf("//\n") ;
  printf("Hello  !\n") ;
  printf("Hello2\n") ;
  int k=0 ;
  setupDDS() ;
  PWMsetup() ;
  
  while(1){
    for(k=0 ; k<5 ; k++){
      long long t1=time_us_64() ;
      long long t2=time_us_64() ;
      int dt=(int) (t2-t1) ;
      printf("zero code t1=%lli t2=%lli t2-t1=%d\n",t1,t2,dt) ;
      busy_wait_us_32(500000) ;
      }

    for(k=0 ; k<5 ; k++){
      long long t1=time_us_64() ;
      for(int n=0 ; n<NN ; n++){
        pwm_set_gpio_level(PWMpin1, DDSsinTab[DDS1phase>>24] );
        pwm_set_gpio_level(PWMpin2, DDStriTab[DDS2phase>>24] );
        DDS1phase += DDS1diff ; 
        DDS2phase += DDS2diff ; 
        }
      long long t2=time_us_64() ;
      printf("Time needed %10.1f ns\n",(t2-t1)*us/ns/NN) ;
      busy_wait_us_32(500000) ;
      }

    for(k=0 ; k<5 ; k++){
      long long t1=time_us_64() ;
      for(int n=0 ; n<NN ; n++){
        }
      long long t2=time_us_64() ;
      printf("loop offset Time needed %10.1f ns\n",(t2-t1)*us/ns/NN) ;
      busy_wait_us_32(500000) ;
      }      

   
    }
  }

 
  

