// uart output


#include "pico/stdlib.h"
#include <stdio.h>
#include <math.h>

#include "pico/time.h"
#include "hardware/irq.h"
#include "hardware/structs/adc.h"
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_pwm\include\hardware\pwm.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_adc\include\hardware\adc.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_gpio\include\hardware\gpio.h>
//#include "hardware/adc.h"

#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_sync\include\hardware\sync.h>

#include "hardware/resets.h"
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_spi\include\hardware\spi.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_spi\spi.c>

#define GPIO4 4      // GPIO4 pin 6
#define PWMpin1 2    // GPIO2 pin 4
#define PWMpin2 3    // GPIO3 pin 5

#define PWMclkDiv 1000
#define repeatPeriod 100

volatile uint32_t DDS1phase=0 ; // DDS phase value
volatile uint32_t DDS1diff=0 ; // DDS phase delta
volatile uint32_t DDS2phase=0 ; // DDS phase value
volatile uint32_t DDS2diff=0 ; // DDS phase delta

volatile int DDSsinTab[256] ;
volatile int DDScosTab[256] ;
volatile int DDStriTab[256] ;

float triangle(float x){
  if(x<=M_PI/2){ return x/(M_PI/2.0) ; }
  if(x<=M_PI){ return triangle(M_PI-x) ; }
  return -triangle(2*M_PI-x) ;
  }

//------------------------------------------------------------------------------------------------------

struct repeating_timer timer;
double fSample ;

bool TimerCallback(struct repeating_timer *t) {
  gpio_put(GPIO4, 1);
  pwm_set_gpio_level(PWMpin1, DDSsinTab[DDS1phase>>24] );
  pwm_set_gpio_level(PWMpin2, DDStriTab[DDS2phase>>24] );
  DDS1phase += DDS1diff ; 
  DDS2phase += DDS2diff ; 
  gpio_put(GPIO4, 0);
  }

void setupDDS(){
  double DDS1frq=20.0 ;
  double DDS2frq=10.0 ;
  DDS1diff=(int)( 4294967296.0*DDS1frq/fSample) ;
  DDS2diff=(int)( 4294967296.0*DDS2frq/fSample) ;

  for(int k=0 ; k<256 ; k++){
    double phi=2*M_PI*k/256.0 ;
    DDScosTab[k]=500+450*cos(phi) ;
    DDSsinTab[k]=500+450*sin(phi) ;
    DDStriTab[k]=500+450*triangle(phi) ;
    }

  add_repeating_timer_us(-repeatPeriod, TimerCallback, NULL, &timer);
  }
  
//----------------------------------------------------------------------------------------  

void PWMsetup(){    
  gpio_set_function(PWMpin1, GPIO_FUNC_PWM);
  gpio_set_function(PWMpin2, GPIO_FUNC_PWM);
  uint slice_num1 = pwm_gpio_to_slice_num(PWMpin1);
  uint slice_num2 = pwm_gpio_to_slice_num(PWMpin2);
  pwm_config config1 = pwm_get_default_config();
  pwm_config_set_clkdiv(&config1, 1.f);
  pwm_init(slice_num1, &config1, true);
  pwm_set_wrap(slice_num1,(PWMclkDiv-1) );
  pwm_set_wrap(slice_num2,(PWMclkDiv-1) );
  float fPWM=125e6/PWMclkDiv ;
  printf("fPWM=%10.1f\n",fPWM) ;
  pwm_set_gpio_level(PWMpin1, 200 );
  pwm_set_gpio_level(PWMpin2, 500 );
  }

void abort(){
  printf("abort..\n") ;
  printf("\n") ;
  while(1){} ;
  }

void gpioTest(){
  gpio_init(PWMpin1);
  gpio_set_dir(PWMpin1, GPIO_OUT);
  gpio_init(PWMpin2);
  gpio_set_dir(PWMpin2, GPIO_OUT);
  while(1){
    gpio_put(PWMpin1, 1);
    gpio_put(PWMpin2, 1);
    sleep_us(1) ;
    gpio_put(PWMpin1, 0);
    gpio_put(PWMpin2, 0);
    sleep_us(1) ;
    }
  }

int main() {
  stdio_init_all();
  sleep_ms(250);
  printf("//\n") ;
  printf("//\n") ;
  printf("Hello dualDDStoPWMv01 !\n") ;
  gpio_init(GPIO4);
  gpio_set_dir(GPIO4, GPIO_OUT);
  PWMsetup() ;
  fSample=1e6/repeatPeriod ;
  printf("fSample=%15.5f\n",fSample) ;
  setupDDS() ;
  while(1){
    tight_loop_contents() ;
    }
  }

