// display via TFT or PC
// see: processing/PCgraphTermV01/...

#include "pico/stdlib.h"
#include <stdio.h>
#include <math.h>

#include "pico/time.h"
#include "hardware/irq.h"
#include "hardware/structs/adc.h"
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_adc\include\hardware\adc.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_gpio\include\hardware\gpio.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_sync\include\hardware\sync.h>
#include "hardware/resets.h"

void restore_interrupts 	( 	uint32_t  	status	) 	;
uint32_t save_and_disable_interrupts 	( 	void  		) ;	

// adc input adc2= GPIO28 pin 34
#define GPIO4 4
int toggle ;

float fSample,tSample ;

/*
#define nSamples 480
#define nDown 1
#define yScale 2
//#define xScale 5
#define meanSample 0
#define withTrigger 0
#define triggerLevel 0
long timeOut=100000 ;
#define printLimit 0
#define ACcouple 1
*/


#define nSamples 500
#define nDown 1
#define yScale 2
//#define xScale 5
#define meanSample 0
#define withTrigger 1
#define triggerLevel 0
long timeOut=100000 ;
#define printLimit 20
#define ACcouple 1


//-----------------------------------------------------------------------------------

#define tftOn 1

// Color definitions
#define ILI9341_BLACK       0x0000      /*   0,   0,   0 */
#define ILI9341_NAVY        0x000F      /*   0,   0, 128 */
#define ILI9341_DARKGREEN   0x03E0      /*   0, 128,   0 */
#define ILI9341_DARKCYAN    0x03EF      /*   0, 128, 128 */
#define ILI9341_MAROON      0x7800      /* 128,   0,   0 */
#define ILI9341_PURPLE      0x780F      /* 128,   0, 128 */
#define ILI9341_OLIVE       0x7BE0      /* 128, 128,   0 */
#define ILI9341_LIGHTGREY   0xC618      /* 192, 192, 192 */
#define ILI9341_DARKGREY    0x7BEF      /* 128, 128, 128 */
#define ILI9341_BLUE        0x001F      /*   0,   0, 255 */
#define ILI9341_GREEN       0x07E0      /*   0, 255,   0 */
#define ILI9341_CYAN        0x07FF      /*   0, 255, 255 */
#define ILI9341_RED         0xF800      /* 255,   0,   0 */
#define ILI9341_MAGENTA     0xF81F      /* 255,   0, 255 */
#define ILI9341_YELLOW      0xFFE0      /* 255, 255,   0 */
#define ILI9341_WHITE       0xFFFF      /* 255, 255, 255 */
#define ILI9341_ORANGE      0xFD20      /* 255, 165,   0 */
#define ILI9341_GREENYELLOW 0xAFE5      /* 173, 255,  47 */
#define ILI9341_PINK        0xF81F

int chrColor = ILI9341_WHITE ; 
int chrBackground = ILI9341_BLACK ; 
int xLast=0 ;
int yLast=0 ;

//-----------------------------------------------------------------------------------

static void approxSleepUs(int n){
  for(int k=0 ; k<n ; k++){
    for( int kk=0 ; kk<5 ; kk++){
      asm volatile("nop");
      }
    }
  }
  
static void approxSleepMs(int n){
  for(int k=0 ; k<n ; k++){
    for( int kk=0 ; kk<10 ; kk++){
      approxSleepUs(1000) ;
      }
    }
  }

static volatile void approxSleepUsTest(){
  printf("atleastSleepUsTest()\n") ;
  save_and_disable_interrupts() ;
  while(1){
    sio_hw->gpio_set =(1ul << GPIO4) ;  
	  approxSleepUs(1000) ;
    //sleep_ms(1) ;
    sio_hw->gpio_clr =(1ul << GPIO4) ;  
    approxSleepUs(1000) ;
  //  printf(".") ;
  }
} 

#include "tftLib1.c"
#include "graphTerm1.c"

int scopeTrace1[nSamples] ;

//------------------------------------------------------------------------------------------------------

void adc_initNew(void) {
    // ADC is in an unknown state. We should start by resetting it
    reset_block(RESETS_RESET_ADC_BITS);
    unreset_block_wait(RESETS_RESET_ADC_BITS);
    // Now turn it back on. Staging of clock etc is handled internally
    adc_hw->cs = ADC_CS_EN_BITS;
    // Internal staging completes in a few cycles, but poll to be sure
    while (!(adc_hw->cs & ADC_CS_READY_BITS)) {
        tight_loop_contents();
    }
}

//----------------------------------------------------------------------------------------  

int sample(){
  int adc=0 ;
  for(int j=0 ; j<nDown ; j++){
    if(meanSample){
      adc += adc_fifo_get_blocking() ;
      }
     else {
      adc = adc_fifo_get_blocking() ;   
      } 
    }
  if(meanSample){ adc=adc/nDown ; }
  return yScale*(adc-2048) ;
  }

void trigger(){
  int lastSample,thisSample ;
  long timer ;
  if(!withTrigger) return ;
  lastSample=sample() ;
  thisSample=sample() ;
  timer=0 ;
  while(1){
    if( (lastSample<=triggerLevel) & (thisSample>=triggerLevel) ) { return ; }
    lastSample=thisSample ;
    thisSample=sample() ;
    timer++ ;
    if(timeOut>0){
      if(timer>=timeOut) { return ; }
      }
    }
  }

void getBuffer() {
  uint32_t state ;
  int adc ;
  printf("R%cgetBuffer()%c\n",'"','"') ;
  adc_fifo_setup(true, false, 0, false, false);
  adc_run(true);
  state=save_and_disable_interrupts () ;
  for (int k = 0; k < 64 ; k++ ){
    adc_fifo_get_blocking();
    }
  trigger() ;  
  float mean=0 ;
  for (int k = 0; k < nSamples ; k++ ){
    float s=sample() ;
    mean += s ;
    scopeTrace1[k]=s ;
    }
  mean=mean/(nSamples) ;
  if(ACcouple){
    for (int k = 0; k < nSamples ; k++ ){
      scopeTrace1[k] -= mean ;
      }
    }
  restore_interrupts (state) ;
 adc_run(false);
  adc_fifo_drain();
  }
  
int yFun(int y){
  return (int) ( tftHeight/2+y/4096.0*tftHeight) ;
  }  

int voltsToY(float v){
  int y=(int)( tftHeight/2+tftHeight/2*v*yScale/(3.3/2.0)) ;
  return y ;
  }

int kToTFT(int k){
  return tftWidth*k/((float)nSamples) ;
  }

#define us 1e-6

int timeToK(float t){
  return (int)( t/tSample) ;
  }

float adcToVolts(float a){
  return 3.3/2.0*a/yScale/2048.0 ;
  }  

float sqr(float x){ return x*x ; }

float kToGraph(int k){
  return dd+(xSize-2*dd)*((float)k)/((float)nSamples) ;
  }

float yTFTtoGraph(int y){
  return dd+(ySize-2*dd)-(ySize-2*dd)*y/((float)tftHeight) ;
  }

void showScope(){
  int m=5 ;
  int tftDisplay=1 ;
  if(tftDisplay){
    int y=voltsToY(0.0) ; brokenLine(0,y,tftWidth,y,m,ILI9341_GREEN) ;
    y=voltsToY(0.5) ; brokenLine(0,y,tftWidth,y,m,ILI9341_GREEN) ;
    y=voltsToY(-0.5) ; brokenLine(0,y,tftWidth,y,m,ILI9341_GREEN) ;
    y=voltsToY(1) ; brokenLine(0,y,tftWidth,y,m,ILI9341_GREEN) ;
    y=voltsToY(-1) ; brokenLine(0,y,tftWidth,y,m,ILI9341_GREEN) ;

    int x=timeToK(0) ; brokenLine(x,0,x,tftHeight,m,ILI9341_GREEN) ;
    x=kToTFT(timeToK(250*us)) ; brokenLine(x,0,x,tftHeight,m,ILI9341_GREEN) ;
    x=kToTFT(timeToK(500*us)) ; brokenLine(x,0,x,tftHeight,m,ILI9341_GREEN) ;
    x=kToTFT(timeToK(750*us)) ; brokenLine(x,0,x,tftHeight,m,ILI9341_GREEN) ;
    cursorX=2 ; cursorY=tftHeight-20 ; tftPrint("Volt") ; 
    cursorX=2 ; cursorY=voltsToY(0.5)-10 ; tftPrint("+0.5") ; 
    cursorX=2 ; cursorY=voltsToY(1.0)-10 ; tftPrint("+1.0") ; 
    cursorX=2 ; cursorY=voltsToY(0)-10 ; tftPrint("0") ; 
    cursorX=2 ; cursorY=voltsToY(-0.5)-10 ; tftPrint("-0.5") ; 
    cursorX=2 ; cursorY=voltsToY(-1.0)-10 ; tftPrint("-1.0") ; 
    cursorX=kToTFT(timeToK(250*us)) ;  cursorY=(tftHeight/2-20) ; tftPrint("250us") ; 
    cursorX=kToTFT(timeToK(500*us)) ;  cursorY=(tftHeight/2-20) ; tftPrint("500us") ; 
    cursorX=kToTFT(timeToK(750*us)) ;  cursorY=(tftHeight/2-20) ; tftPrint("750us") ;
   // cursorX=xScale*timeToK(910*us) ;  cursorY=(tftHeight/2-20) ; tftPrint("us") ;
    for(int k=1 ; k<nSamples ; k++){
      tftLine( kToTFT(k-1), yFun(scopeTrace1[k-1]) ,kToTFT(k),yFun(scopeTrace1[k]),0x0FFFF) ;
      }
    }
  
  int graphDisplay=1 ;  
  if(graphDisplay){
    printf("B\n") ; sleep_ms(100) ;
    printf("H16;\n") ;
    printf("W2;\n") ;
    printf("M10;25;\n") ; printf("T%cPCgraphTerm V0.1%c\n",'"','"') ;
    float endTime=nSamples*tSample ;
    printf("M200;25;") ; printf("T%cendT=%8.1f us%c",'"',endTime/1e-6,'"') ;
    graphFrame() ;

    char sbuf[200] ;
    sprintf(sbuf,"endT=%8.1f us",endTime/1e-6) ;
    cursorX=200 ; cursorY=tftHeight-20 ; tftPrint(sbuf) ; 

    int y=voltsToY(0.0) ;
    BLine(dd,yTFTtoGraph(y),xSize-dd,yTFTtoGraph(y),15 ) ;
    MMove(dd+5,yTFTtoGraph(y)-5 ) ;
    TText("0.0 V") ;

    y=voltsToY(0.5) ;
    BLine(dd,yTFTtoGraph(y),xSize-dd,yTFTtoGraph(y),15 ) ;
    MMove(dd+5,yTFTtoGraph(y)-5 ) ;
    TText("0.5 V") ;

    y=voltsToY(-0.5) ;
    BLine(dd,yTFTtoGraph(y),xSize-dd,yTFTtoGraph(y),15 ) ;
    MMove(dd+5,yTFTtoGraph(y)-5 ) ;
    TText("-0.5 V") ;

    y=voltsToY(1) ; 
    BLine(dd,yTFTtoGraph(y),xSize-dd,yTFTtoGraph(y),15 ) ;
    MMove(dd+5,yTFTtoGraph(y)-5 ) ;
    TText("1.0 V") ;

    y=voltsToY(-1) ;
    BLine(dd,yTFTtoGraph(y),xSize-dd,yTFTtoGraph(y),15 ) ;
    MMove(dd+5,yTFTtoGraph(y)-5 ) ;
    TText("-1.0 V") ;

    int x=timeToK(0) ;
    BLine(kToGraph(x),dd,kToGraph(x),ySize-dd,15) ;

    x=kToGraph(timeToK(250*us)) ; 
    BLine(x,dd,x,ySize-dd,15) ;
    MMove(x,yTFTtoGraph(voltsToY(0.01) ) ) ;
    TText("250us") ;

    x=kToGraph(timeToK(500*us)) ; 
    BLine(x,dd,x,ySize-dd,15) ;
    MMove(x,yTFTtoGraph(voltsToY(0.01) ) ) ;
    TText("500us") ;

    x=kToGraph(timeToK(750*us)) ;
    BLine(x,dd,x,ySize-dd,15) ;
    MMove(x,yTFTtoGraph(voltsToY(0.01) ) ) ;
    TText("750us") ;

    SStroke(250,250,250) ;

    for(int k=0 ; k<nSamples ; k++){
      if(k==0){ MMove(kToGraph(k),yTFTtoGraph(yFun(scopeTrace1[k])) ) ; }
       else { DDraw(kToGraph(k),yTFTtoGraph(yFun(scopeTrace1[k])) ) ; }
      }
    }
  /*
  for(int k=0 ; k<nSample/xScale ; k++){  
    if(k<printLimit){
      printf("%5d :: %10.3f\n",k,(float)scopeTrace1[k]) ;
      }
    }
  */  
  //printf("fSample=%10.1f Hz\n",fSample) ;
  float vMean=0.0 ;
  float vMax=-1e5 ;
  float vMin=1e5 ;
  float vRMS=0.0 ;
  for(int k=0 ; k<nSamples ; k++){
    float v=adcToVolts(scopeTrace1[k]) ;
    vMean += v ;
    if(v>vMax){ vMax=v ; }
    if(v<vMin){ vMin=v ; }
    vRMS+=sqr(v) ;
    }
  vRMS=sqrt(vRMS/nSamples) ;
  vMean=vMean/nSamples ; 
  char sbuf[200] ;  
  sprintf(sbuf,"Vmean=%10.4f V",vMean) ; printf("R%c%s%c\n",'"',sbuf,'"') ;

  sprintf(sbuf,"Vrms =%10.4f V",vRMS) ; printf("R%c%s%c\n",'"',sbuf,'"') ;
  sprintf(sbuf,"Vmax =%10.4f V",vMax) ; printf("R%c%s%c\n",'"',sbuf,'"') ;
  sprintf(sbuf,"Vmin =%10.4f V",vMin) ; printf("R%c%s%c\n",'"',sbuf,'"') ;
  sprintf(sbuf,"Vpp  =%10.4f V",vMax-vMin) ; printf("R%c%s%c\n",'"',sbuf,'"') ;
  }

int adc_read2(void) {
    hw_set_bits(&adc_hw->cs, ADC_CS_START_ONCE_BITS);
    while (!(adc_hw->cs & ADC_CS_READY_BITS)){}
       
    return adc_hw->result;
    }


int repeatFired=0 ;
bool repeating_timer_callback(struct repeating_timer *t) {
  repeatFired=1 ;
  return true;
}

void rollDisplay(){
  printf("rollDisplay()\n") ;
  printf("nSamples=%d\n",nSamples) ;
  //adc_fifo_setup(true, false, 0, false, false);
  // adc_fifo_setup(false, false, 0, false, false);
  // adc_run(true);

  struct repeating_timer timer;
  // Negative delay so means we will call repeating_timer_callback, and call it again
  // 300ms later regardless of how long the callback took to execute
    add_repeating_timer_ms(-100, repeating_timer_callback, NULL, &timer);
    //cancelled = cancel_repeating_timer(&timer);
   /*
  while(1){
    float s=0 ;
      for(int k=0 ; k<mSamples ; k++){
        while(repeatFired==0){} ;
        repeatFired=0 ;
        s=adc_read()-2048 ;
        scopeTrace1[k]=s ;
        }
      LCD_Clear(0x0000);
      for(int k=1 ; k<mSamples ; k++){
      tftLine( xScale*(k-1), yFun(scopeTrace1[k-1]) ,xScale*k,yFun(scopeTrace1[k]),0x0FFFF) ;
      }  
    }
*/

  while(1){
    float s=0 ;
   // for(int k=0 ; k<1 ; k++){
    // adc_fifo_drain();
    
      //s=sample() ;
      while(repeatFired==0){} ;
      repeatFired=0 ;
      s=adc_read()-2048 ;
     // printf("s=%10.3f\n",s) ;

     // adc_fifo_drain();
   //   }
  // printf("s=%10.4f\n",s) ;
  // tftLine(10,10,100,100,0x0FFFF) ;

    for(int k=1 ; k<nSamples ; k++){
      tftLine( kToTFT(k-1), yFun(scopeTrace1[k-1]) ,kToTFT(k),yFun(scopeTrace1[k]),0x0) ;
      scopeTrace1[k-1]=scopeTrace1[k] ;
      }
    scopeTrace1[nSamples-1]=s ;
    for(int k=1 ; k<nSamples ; k++){
      tftLine( kToTFT(k-1), yFun(scopeTrace1[k-1]) ,kToTFT(k),yFun(scopeTrace1[k]),0x0FFFF) ;
      //printf("tftLine(%d,%d,%d,%d,%d)\n",xScale*(k-1), yFun(scopeTrace1[k-1]) ,xScale*k,yFun(scopeTrace1[k]),0x0FFFF) ;
      // tftLine( (k-1), k ,k,k,0x0FFFF) ;
      // printf("tftLine(%d,%d,%d,%d,%d)\n",(k-1), k ,k,k,0x0FFFF) ;
      }    
    }
  } 


void showBuffer(){
  // fillRectOssi2(0, 0, xSize, ySize, ILI9341_BLACK ) ;
  getBuffer() ;
  tftClear(ILI9341_BLACK ) ;
  tftLine(0,0,0,320,ILI9341_WHITE) ;
  showScope() ; 
  approxSleepMs(2000) ;
  }  

//----------------------------------------------------------------------------------------  

int main() {
  stdio_init_all();
  gpio_init(GPIO4);
  gpio_set_dir(GPIO4, GPIO_OUT);

  // PWMsetup() ;
  fSample=500000.0/nDown ;
  tSample=1.0/fSample ;

  printf("fSample=%15.5f\n",fSample) ;
 

  tftSetup() ;
    
  approxSleepMs(200);
  printf("//\n") ;
  printf("//\n") ;
  printf("Hello 1... !\n") ;
  
  toggle=0 ;
   
  
  adc_initNew();
  // adc_init();
  // Make sure GPIO is high-impedance, no pullups etc

  
// Select ADC input 2 (GPIO28)
  adc_gpio_init(28);
  adc_select_input(2);
  printf("Hello 2... !\n") ;
  while (1){
    showBuffer() ;
    //rollDisplay() ;
    }
 

  }

