// console hardware uart 115200bit/sec

#include "pico/stdlib.h"
#include <stdio.h>
#include <math.h>

#include "pico/time.h"
#include "hardware/irq.h"
#include "hardware/structs/adc.h"
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_pwm\include\hardware\pwm.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_gpio\include\hardware\gpio.h>

#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_sync\include\hardware\sync.h>

#include "hardware/resets.h"
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_spi\include\hardware\spi.h>
#include <C:\abOssmann\PiPico\installation\Pico\pico-sdk\src\rp2_common\hardware_spi\spi.c>
#include "hardware/regs/rosc.h"

/*                         LDAC pin 5 connected to GND
   GPIO 16 (pin 21) MISO
   GPIO 17 (pin 22) CS     CS  pin 2
   GPIO 18 (pin 24) SCK    SCK pin 3
   GPIO 19 (pin 25) MOSI   SDI pin 4
   3.3v (pin 36) -> VCC    VDD pin 1 (2.7V..5.5V)
   GND (pin 38)  -> GND    VSS pin 7
*/
#define PWMclkDiv 625
#define PWM_PIN1 2    // GPIO2 pin 4
#define PWM_PIN2 3    // GPIO3 pin 5
#define GPIO4 4     // pin 6    

#define TFT_MISO 16 
#define TFT_CS 17 
#define TFT_SCK 18
#define TFT_MOSI 19 

#define nSHDN 1 
#define nGA   1 

float Ohm=1 ;
float mW=1e-3 ;

float twopi = 2*M_PI ;
int DDStable[4096] ;

float sampleFrequency  ; 
#define Tone1FRQ (1e3)
int Tone1ddsFrequency ;
int Tone1ddsPhase ;


#define Tone2FRQ (2.5e3)
int Tone2ddsFrequency ;
int Tone2ddsPhase ;

//-----------------------------------------------------------------------------------

static void approxSleepUs(int n){
  for(int k=0 ; k<n ; k++){
    for( int kk=0 ; kk<5 ; kk++){
      asm volatile("nop");
      }
    }
  }
  
static void approxSleepMs(int n){
  for(int k=0 ; k<n ; k++){
    for( int kk=0 ; kk<10 ; kk++){
      approxSleepUs(1000) ;
      }
    }
  }

static volatile void approxSleepUsTest(){
  printf("atleastSleepUsTest()\n") ;
  save_and_disable_interrupts() ;
  while(1){
    sio_hw->gpio_set =(1ul << GPIO4) ;  
	  approxSleepUs(1000) ;
    //sleep_ms(1) ;
    sio_hw->gpio_clr =(1ul << GPIO4) ;  
    approxSleepUs(1000) ;
  //  printf(".") ;
  }
} 

float fabs1(float x){
  if(x>0){ return x ; }
  return -x ;
  }

//------------------------------------------------------------------------------------------------------
#define spiFIFOsize 256
#define spiFIFOmask (spiFIFOsize-1)

int spiFIFO[spiFIFOsize] ;
int spiFIFOinPtr ;
int spiFIFOoutPtr ;

void spiFIFOinit(){
  spiFIFOinPtr=0 ;
  spiFIFOoutPtr=1 ;
  }

uint16_t spiBuf16[2];
volatile int spiData ;
volatile int spiFetched ;

int spiFIFOfull(){
  int state ;
  int flag ;
  state=save_and_disable_interrupts () ;
  flag= spiFIFOinPtr==spiFIFOoutPtr ;
  restore_interrupts (state) ;
  return flag ;
  }

void PWMirq() {
  gpio_put(GPIO4, 1);
  spiBuf16[0] =spiData ;
  // Clear the interrupt flag that brought us here
  pwm_clear_irq(pwm_gpio_to_slice_num(PWM_PIN1));
  spi_write16_blocking(spi_default, spiBuf16, 1);
  spiData=spiFIFO[spiFIFOoutPtr] ;
  spiFIFOoutPtr=(spiFIFOoutPtr+1) & spiFIFOmask ;
  gpio_put(GPIO4, 0);
  }

//-----------------------------------------------------------------------------------

void PWMsetup(){    
  gpio_set_function(PWM_PIN1, GPIO_FUNC_PWM);
  gpio_set_function(PWM_PIN2, GPIO_FUNC_PWM);
   // Figure out which slice we just connected to the pin
  uint slice_num1 = pwm_gpio_to_slice_num(PWM_PIN1);
  uint slice_num2 = pwm_gpio_to_slice_num(PWM_PIN2);
  // Mask our slice's IRQ output into the PWM block's single interrupt line,
  // and register our interrupt handler
  pwm_clear_irq(slice_num1);
  pwm_set_irq_enabled(slice_num1, true);
  irq_set_exclusive_handler(PWM_IRQ_WRAP, PWMirq);
  irq_set_enabled(PWM_IRQ_WRAP, true);
  // Get some sensible defaults for the slice configuration. 
  pwm_config config1 = pwm_get_default_config();
  // Set divider, reduces counter clock to sysclock/this value
  pwm_config_set_clkdiv(&config1, 1.f);
  pwm_config_set_output_polarity (&config1,1,0) ; // invert A channel

  // Load the configuration into our PWM slice, and set it running.
  pwm_init(slice_num1, &config1, true);
  // Set period of 125 cycles (0 to 124 inclusive)
  //pwm_set_wrap(slice_num1,124 );
 
  pwm_set_wrap(slice_num1,(PWMclkDiv-1) );
  sampleFrequency=125e6/PWMclkDiv/1.0 ;

  pwm_set_gpio_level(PWM_PIN1, 500);
  pwm_set_gpio_level(PWM_PIN2, 400);

  printf("sampleFrequency=%15.5f\n",sampleFrequency) ;
  printf("PWM_PIN1=%d\n",PWM_PIN1) ;
  printf("slice_num1=%d\n",slice_num1) ;
  
  printf("PWM_PIN2=%d\n",PWM_PIN2) ;
  printf("slice_num2=%d\n",slice_num2) ;

  printf("config1=%d\n",config1) ;
  printf("\n") ;
  }

void abort(){
  printf("abort..\n") ;
  printf("\n") ;
  while(1){} ;
  }

inline void putSample(int v){
  while(spiFIFOfull() ){}
  spiFIFO[spiFIFOinPtr]=v ;
  spiFIFOinPtr=(spiFIFOinPtr+1) & spiFIFOmask ;
  }

int main() {
  stdio_init_all();
  gpio_init(GPIO4);
  gpio_set_dir(GPIO4, GPIO_OUT);
  printf("//\n") ;
  printf("//\n") ;
  printf("Hello twoToneSiggenMCP4821v01.c !\n") ;

  gpio_init(TFT_CS);
  gpio_set_dir(TFT_CS, GPIO_OUT);
  gpio_put(TFT_CS, 1);
  
  spiFIFOinit() ;
  spi_init(spi_default,  24000000);
  // gpio_set_function(PICO_DEFAULT_SPI_RX_PIN, GPIO_FUNC_SPI);
  gpio_set_function(PICO_DEFAULT_SPI_SCK_PIN, GPIO_FUNC_SPI);
  gpio_set_function(PICO_DEFAULT_SPI_TX_PIN, GPIO_FUNC_SPI);
  spi_set_format(spi_default,16,0,0,SPI_MSB_FIRST) ;

  PWMsetup() ;
 
  int M=(int) (sqrt(2)*4096/2.048*sqrt(1*mW*50*Ohm)) ;
  printf("M=%d\n",M) ;
  for(int k=0 ; k<4096 ; k++){
    DDStable[k]=M*sin(twopi*k/4096.0) ;
    }
 
  Tone1ddsPhase=0 ;
  Tone1ddsFrequency=Tone1FRQ/sampleFrequency*(double)(0x100000000L) ;

  Tone2ddsPhase=0 ;
  Tone2ddsFrequency=Tone2FRQ/sampleFrequency*(double)(0x100000000L) ;
 
  while(1){
    int nextVal= (2048+DDStable[ (Tone1ddsPhase >> 20 ) & 0xFFF ] 
               + DDStable[ (Tone2ddsPhase >> 20 ) & 0xFFF ] ) 
               | (nSHDN<<12) | (nGA<<13) ;
    Tone1ddsPhase=Tone1ddsPhase+Tone1ddsFrequency ;
    Tone2ddsPhase=Tone2ddsPhase+Tone2ddsFrequency ;
    putSample(nextVal) ;
    }
 
  }

